@@ -26,6 +26,14 @@
 extern "C" {
 #endif
 
+#define __ARB_VERSION 2
+#define __ARB_VERSION_MINOR 20
+#define __ARB_VERSION_PATCHLEVEL 0
+#define ARB_VERSION "2.20.0"
+#define __ARB_RELEASE (__ARB_VERSION * 10000 + \
+                         __ARB_VERSION_MINOR * 100 + \
+                         __ARB_VERSION_PATCHLEVEL)
+
 ARB_DLL extern const char * arb_version;
 double arb_test_multiplier(void);
 
@@ -56,26 +64,8 @@ arb_init(arb_t x)
 
 void arb_clear(arb_t x);
 
-ARB_INLINE arb_ptr
-_arb_vec_init(slong n)
-{
-    slong i;
-    arb_ptr v = (arb_ptr) flint_malloc(sizeof(arb_struct) * n);
-
-    for (i = 0; i < n; i++)
-        arb_init(v + i);
-
-    return v;
-}
-
-ARB_INLINE void
-_arb_vec_clear(arb_ptr v, slong n)
-{
-    slong i;
-    for (i = 0; i < n; i++)
-        arb_clear(v + i);
-    flint_free(v);
-}
+arb_ptr _arb_vec_init(slong n);
+void _arb_vec_clear(arb_ptr v, slong n);
 
 ARB_INLINE arf_ptr arb_mid_ptr(arb_t z) { return arb_midref(z); }
 ARB_INLINE mag_ptr arb_rad_ptr(arb_t z) { return arb_radref(z); }
@@ -99,6 +89,7 @@ arb_equal_si(const arb_t x, slong y)
     return arf_equal_si(arb_midref(x), y) && mag_is_zero(arb_radref(x));
 }
 
+/* implementations are in arb/richcmp.c */
 int arb_eq(const arb_t x, const arb_t y);
 int arb_ne(const arb_t x, const arb_t y);
 int arb_lt(const arb_t x, const arb_t y);
@@ -106,12 +97,7 @@ int arb_le(const arb_t x, const arb_t y);
 int arb_gt(const arb_t x, const arb_t y);
 int arb_ge(const arb_t x, const arb_t y);
 
-ARB_INLINE void
-arb_zero(arb_t x)
-{
-    arf_zero(arb_midref(x));
-    mag_zero(arb_radref(x));
-}
+void arb_zero(arb_t x);
 
 ARB_INLINE int
 arb_is_zero(const arb_t x)
@@ -141,18 +127,25 @@ arb_zero_pm_inf(arb_t x)
 }
 
 ARB_INLINE void
-arb_indeterminate(arb_t x)
+arb_zero_pm_one(arb_t x)
 {
-    arf_nan(arb_midref(x));
-    mag_inf(arb_radref(x));
+    arf_zero(arb_midref(x));
+    mag_one(arb_radref(x));
 }
 
-ARB_INLINE int
-arb_is_finite(const arb_t x)
+ARB_INLINE void
+arb_unit_interval(arb_t x)
 {
-    return arf_is_finite(arb_midref(x)) && mag_is_finite(arb_radref(x));
+    arf_one(arb_midref(x));
+    mag_one(arb_radref(x));
+    ARF_EXP(arb_midref(x))--;
+    MAG_EXP(arb_radref(x))--;
 }
 
+void arb_indeterminate(arb_t x);
+
+int arb_is_finite(const arb_t x);
+
 void arb_set(arb_t x, const arb_t y);
 
 ARB_INLINE void
@@ -174,8 +167,9 @@ void arb_neg_round(arb_t x, const arb_t y, slong prec);
 void arb_abs(arb_t y, const arb_t x);
 
 void arb_sgn(arb_t res, const arb_t x);
+int arb_sgn_nonzero(const arb_t x);
 
-void _arb_digits_round_inplace(char * s, mp_bitcnt_t * shift, fmpz_t error, slong n, arf_rnd_t rnd);
+void _arb_digits_round_inplace(char * s, flint_bitcnt_t * shift, fmpz_t error, slong n, arf_rnd_t rnd);
 
 int arb_set_str(arb_t res, const char * inp, slong prec);
 
@@ -193,33 +187,13 @@ arb_set_arf(arb_t x, const arf_t y)
     mag_zero(arb_radref(x));
 }
 
-ARB_INLINE void
-arb_set_si(arb_t x, slong y)
-{
-    arf_set_si(arb_midref(x), y);
-    mag_zero(arb_radref(x));
-}
+void arb_set_si(arb_t x, slong y);
 
-ARB_INLINE void
-arb_set_ui(arb_t x, ulong y)
-{
-    arf_set_ui(arb_midref(x), y);
-    mag_zero(arb_radref(x));
-}
+void arb_set_ui(arb_t x, ulong y);
 
-ARB_INLINE void
-arb_set_d(arb_t x, double y)
-{
-    arf_set_d(arb_midref(x), y);
-    mag_zero(arb_radref(x));
-}
+void arb_set_d(arb_t x, double y);
 
-ARB_INLINE void
-arb_set_fmpz(arb_t x, const fmpz_t y)
-{
-    arf_set_fmpz(arb_midref(x), y);
-    mag_zero(arb_radref(x));
-}
+void arb_set_fmpz(arb_t x, const fmpz_t y);
 
 ARB_INLINE void
 arb_set_fmpz_2exp(arb_t x, const fmpz_t y, const fmpz_t exp)
@@ -238,12 +212,7 @@ arb_is_one(const arb_t f)
     return arf_is_one(arb_midref(f)) && mag_is_zero(arb_radref(f));
 }
 
-ARB_INLINE void
-arb_one(arb_t f)
-{
-    arf_one(arb_midref(f));
-    mag_zero(arb_radref(f));
-}
+void arb_one(arb_t f);
 
 void arb_fprint(FILE * file, const arb_t x);
 
@@ -269,12 +238,7 @@ arb_printn(const arb_t x, slong digits, ulong flags)
     arb_fprintn(stdout, x, digits, flags);
 }
 
-ARB_INLINE void
-arb_mul_2exp_si(arb_t y, const arb_t x, slong e)
-{
-    arf_mul_2exp_si(arb_midref(y), arb_midref(x), e);
-    mag_mul_2exp_si(arb_radref(y), arb_radref(x), e);
-}
+void arb_mul_2exp_si(arb_t y, const arb_t x, slong e);
 
 ARB_INLINE void
 arb_mul_2exp_fmpz(arb_t y, const arb_t x, const fmpz_t e)
@@ -291,80 +255,23 @@ arb_is_int(const arb_t x)
 }
 
 ARB_INLINE int
-arb_contains_zero(const arb_t x)
+arb_is_int_2exp_si(const arb_t x, slong e)
 {
-    return arf_cmpabs_mag(arb_midref(x), arb_radref(x)) <= 0;
+    return mag_is_zero(arb_radref(x)) &&
+           arf_is_int_2exp_si(arb_midref(x), e);
 }
 
-ARB_INLINE int
-arb_is_nonzero(const arb_t x)
-{
-    return !arb_contains_zero(x);
-}
-
-ARB_INLINE int
-arb_is_positive(const arb_t x)
-{
-    return (arf_sgn(arb_midref(x)) > 0) &&
-        (arf_mag_cmpabs(arb_radref(x), arb_midref(x)) < 0) &&
-         !arf_is_nan(arb_midref(x));
-}
-
-ARB_INLINE int
-arb_is_nonnegative(const arb_t x)
-{
-    return (arf_sgn(arb_midref(x)) >= 0) &&
-        (arf_mag_cmpabs(arb_radref(x), arb_midref(x)) <= 0) &&
-         !arf_is_nan(arb_midref(x));
-}
-
-ARB_INLINE int
-arb_is_negative(const arb_t x)
-{
-    return (arf_sgn(arb_midref(x)) < 0) &&
-        (arf_mag_cmpabs(arb_radref(x), arb_midref(x)) < 0) &&
-         !arf_is_nan(arb_midref(x));
-}
-
-ARB_INLINE int
-arb_is_nonpositive(const arb_t x)
-{
-    return (arf_sgn(arb_midref(x)) <= 0) &&
-        (arf_mag_cmpabs(arb_radref(x), arb_midref(x)) <= 0) &&
-         !arf_is_nan(arb_midref(x));
-}
-
-ARB_INLINE int
-arb_contains_negative(const arb_t x)
-{
-    return (arf_sgn(arb_midref(x)) < 0) ||
-        (arf_mag_cmpabs(arb_radref(x), arb_midref(x)) > 0)
-        || arf_is_nan(arb_midref(x));
-}
-
-ARB_INLINE int
-arb_contains_nonpositive(const arb_t x)
-{
-    return (arf_sgn(arb_midref(x)) <= 0) ||
-        (arf_mag_cmpabs(arb_radref(x), arb_midref(x)) >= 0)
-        || arf_is_nan(arb_midref(x));
-}
-
-ARB_INLINE int
-arb_contains_positive(const arb_t x)
-{
-    return (arf_sgn(arb_midref(x)) > 0) ||
-        (arf_mag_cmpabs(arb_radref(x), arb_midref(x)) > 0)
-        || arf_is_nan(arb_midref(x));
-}
-
-ARB_INLINE int
-arb_contains_nonnegative(const arb_t x)
-{
-    return (arf_sgn(arb_midref(x)) >= 0) ||
-        (arf_mag_cmpabs(arb_radref(x), arb_midref(x)) >= 0)
-        || arf_is_nan(arb_midref(x));
-}
+/* implementations are in arb/richcmp.c */
+int arb_contains_zero(const arb_t x);
+int arb_is_nonzero(const arb_t x);
+int arb_is_positive(const arb_t x);
+int arb_is_nonnegative(const arb_t x);
+int arb_is_negative(const arb_t x);
+int arb_is_nonpositive(const arb_t x);
+int arb_contains_negative(const arb_t x);
+int arb_contains_nonpositive(const arb_t x);
+int arb_contains_positive(const arb_t x);
+int arb_contains_nonnegative(const arb_t x);
 
 void arb_get_mag_lower(mag_t z, const arb_t x);
 
@@ -393,57 +300,12 @@ arb_get_rad_arb(arb_t z, const arb_t x)
     mag_zero(arb_radref(z));
 }
 
-ARB_INLINE void
-arb_get_abs_ubound_arf(arf_t u, const arb_t x, slong prec)
-{
-    arf_t t;
-    arf_init_set_mag_shallow(t, arb_radref(x));
+void arb_get_abs_ubound_arf(arf_t u, const arb_t x, slong prec);
+void arb_get_abs_lbound_arf(arf_t u, const arb_t x, slong prec);
+void arb_get_ubound_arf(arf_t u, const arb_t x, slong prec);
+void arb_get_lbound_arf(arf_t u, const arb_t x, slong prec);
 
-    if (arf_sgn(arb_midref(x)) < 0)
-        arf_sub(u, arb_midref(x), t, prec, ARF_RND_UP);
-    else
-        arf_add(u, arb_midref(x), t, prec, ARF_RND_UP);
-
-    arf_abs(u, u);
-}
-
-ARB_INLINE void
-arb_get_abs_lbound_arf(arf_t u, const arb_t x, slong prec)
-{
-    arf_t t;
-    arf_init_set_mag_shallow(t, arb_radref(x));
-
-    if (arf_sgn(arb_midref(x)) > 0)
-    {
-        arf_sub(u, arb_midref(x), t, prec, ARF_RND_UP);
-    }
-    else
-    {
-        arf_add(u, arb_midref(x), t, prec, ARF_RND_UP);
-        arf_neg(u, u);
-    }
-
-    if (arf_sgn(u) < 0)
-        arf_zero(u);
-}
-
-ARB_INLINE void
-arb_get_ubound_arf(arf_t u, const arb_t x, long prec)
-{
-    arf_t t;
-    arf_init_set_mag_shallow(t, arb_radref(x));
-
-    arf_add(u, arb_midref(x), t, prec, ARF_RND_CEIL);
-}
-
-ARB_INLINE void
-arb_get_lbound_arf(arf_t u, const arb_t x, long prec)
-{
-    arf_t t;
-    arf_init_set_mag_shallow(t, arb_radref(x));
-
-    arf_sub(u, arb_midref(x), t, prec, ARF_RND_FLOOR);
-}
+void arb_nonnegative_part(arb_t res, const arb_t x);
 
 slong arb_rel_error_bits(const arb_t x);
 
@@ -453,6 +315,8 @@ arb_rel_accuracy_bits(const arb_t x)
     return -arb_rel_error_bits(x);
 }
 
+slong arb_rel_one_accuracy_bits(const arb_t x);
+
 ARB_INLINE slong
 arb_bits(const arb_t x)
 {
@@ -469,6 +333,8 @@ void arb_randtest(arb_t x, flint_rand_t state, slong prec, slong mag_bits);
 
 void arb_randtest_special(arb_t x, flint_rand_t state, slong prec, slong mag_bits);
 
+void arb_urandom(arb_t x, flint_rand_t state, slong prec);
+
 void arb_add_error_arf(arb_t x, const arf_t err);
 
 void arb_add_error_2exp_si(arb_t x, slong err);
@@ -497,6 +363,8 @@ int arb_overlaps(const arb_t x, const arb_t y);
 
 int arb_contains(const arb_t x, const arb_t y);
 
+int arb_contains_interior(const arb_t x, const arb_t y);
+
 int arb_contains_int(const arb_t x);
 
 void arb_get_interval_fmpz_2exp(fmpz_t a, fmpz_t b, fmpz_t exp, const arb_t x);
@@ -513,6 +381,9 @@ void arb_set_interval_mpfr(arb_t x, const mpfr_t a, const mpfr_t b, slong prec);
 void arb_get_interval_arf(arf_t a, arf_t b, const arb_t x, slong prec);
 void arb_get_interval_mpfr(mpfr_t a, mpfr_t b, const arb_t x);
 
+void arb_set_interval_mag(arb_t res, const mag_t a, const mag_t b, slong prec);
+void arb_set_interval_neg_pos_mag(arb_t res, const mag_t a, const mag_t b, slong prec);
+
 void arb_union(arb_t z, const arb_t x, const arb_t y, slong prec);
 int arb_intersection(arb_t z, const arb_t x, const arb_t y, slong prec);
 void arb_get_rand_fmpq(fmpq_t q, flint_rand_t state, const arb_t x, slong bits);
@@ -554,6 +425,27 @@ void arb_submul_si(arb_t z, const arb_t x, slong y, slong prec);
 void arb_submul_ui(arb_t z, const arb_t x, ulong y, slong prec);
 void arb_submul_fmpz(arb_t z, const arb_t x, const fmpz_t y, slong prec);
 
+void arb_dot_simple(arb_t res, const arb_t initial, int subtract,
+    arb_srcptr x, slong xstep, arb_srcptr y, slong ystep, slong len, slong prec);
+void arb_dot_precise(arb_t res, const arb_t initial, int subtract,
+    arb_srcptr x, slong xstep, arb_srcptr y, slong ystep, slong len, slong prec);
+void arb_dot(arb_t res, const arb_t initial, int subtract,
+    arb_srcptr x, slong xstep, arb_srcptr y, slong ystep, slong len, slong prec);
+
+void arb_approx_dot(arb_t res, const arb_t initial, int subtract,
+    arb_srcptr x, slong xstep, arb_srcptr y, slong ystep, slong len, slong prec);
+
+void arb_dot_ui(arb_t res, const arb_t initial, int subtract,
+    arb_srcptr x, slong xstep, const ulong * y, slong ystep, slong len, slong prec);
+void arb_dot_si(arb_t res, const arb_t initial, int subtract,
+    arb_srcptr x, slong xstep, const slong * y, slong ystep, slong len, slong prec);
+void arb_dot_uiui(arb_t res, const arb_t initial, int subtract,
+    arb_srcptr x, slong xstep, const ulong * y, slong ystep, slong len, slong prec);
+void arb_dot_siui(arb_t res, const arb_t initial, int subtract,
+    arb_srcptr x, slong xstep, const ulong * y, slong ystep, slong len, slong prec);
+void arb_dot_fmpz(arb_t res, const arb_t initial, int subtract,
+    arb_srcptr x, slong xstep, const fmpz * y, slong ystep, slong len, slong prec);
+
 void arb_div(arb_t z, const arb_t x, const arb_t y, slong prec);
 void arb_div_arf(arb_t z, const arb_t x, const arf_t y, slong prec);
 void arb_div_si(arb_t z, const arb_t x, slong y, slong prec);
@@ -603,6 +495,7 @@ void arb_log_ui(arb_t z, ulong x, slong prec);
 void arb_log_fmpz(arb_t z, const fmpz_t x, slong prec);
 void arb_log1p(arb_t r, const arb_t z, slong prec);
 void arb_log_base_ui(arb_t res, const arb_t x, ulong b, slong prec);
+void arb_log_hypot(arb_t res, const arb_t x, const arb_t y, slong prec);
 void arb_exp(arb_t z, const arb_t x, slong prec);
 void arb_expm1(arb_t z, const arb_t x, slong prec);
 void arb_exp_invexp(arb_t z, arb_t w, const arb_t x, slong prec);
@@ -623,6 +516,7 @@ void arb_sin_cos_pi_fmpq(arb_t s, arb_t c, const fmpq_t x, slong prec);
 void arb_sin_pi_fmpq(arb_t s, const fmpq_t x, slong prec);
 void arb_cos_pi_fmpq(arb_t c, const fmpq_t x, slong prec);
 void arb_sinc(arb_t z, const arb_t x, slong prec);
+void arb_sinc_pi(arb_t z, const arb_t x, slong prec);
 void arb_sinh(arb_t z, const arb_t x, slong prec);
 void arb_cosh(arb_t z, const arb_t x, slong prec);
 void arb_sinh_cosh(arb_t s, arb_t c, const arb_t x, slong prec);
@@ -636,6 +530,13 @@ void arb_acos(arb_t z, const arb_t x, slong prec);
 void arb_atanh(arb_t z, const arb_t x, slong prec);
 void arb_asinh(arb_t z, const arb_t x, slong prec);
 void arb_acosh(arb_t z, const arb_t x, slong prec);
+
+void arb_sec(arb_t res, const arb_t x, slong prec);
+void arb_csc(arb_t res, const arb_t x, slong prec);
+void arb_csc_pi(arb_t res, const arb_t x, slong prec);
+void arb_sech(arb_t res, const arb_t x, slong prec);
+void arb_csch(arb_t res, const arb_t x, slong prec);
+
 void arb_fac_ui(arb_t z, ulong n, slong prec);
 void arb_doublefac_ui(arb_t z, ulong n, slong prec);
 void arb_bin_ui(arb_t z, const arb_t n, ulong k, slong prec);
@@ -704,6 +605,7 @@ void arb_power_sum_vec(arb_ptr res, const arb_t a, const arb_t b, slong len, slo
 void arb_bell_sum_taylor(arb_t res, const fmpz_t n, const fmpz_t a, const fmpz_t b, const fmpz_t mmag, slong prec);
 void arb_bell_sum_bsplit(arb_t res, const fmpz_t n, const fmpz_t a, const fmpz_t b, const fmpz_t mmag, slong prec);
 void arb_bell_fmpz(arb_t res, const fmpz_t n, slong prec);
+void arb_bell_ui(arb_t res, ulong n, slong prec);
 
 void arb_euler_number_fmpz(arb_t res, const fmpz_t n, slong prec);
 void arb_euler_number_ui(arb_t res, ulong n, slong prec);
@@ -711,6 +613,8 @@ void arb_euler_number_ui(arb_t res, ulong n, slong prec);
 void arb_partitions_fmpz(arb_t res, const fmpz_t n, slong prec);
 void arb_partitions_ui(arb_t res, ulong n, slong prec);
 
+void arb_lambertw(arb_t res, const arb_t x, int flags, slong prec);
+
 ARB_INLINE void
 arb_sqr(arb_t res, const arb_t val, slong prec)
 {
@@ -748,6 +652,18 @@ _arb_vec_entry_ptr(arb_ptr vec, slong i)
     return vec + i;
 }
 
+ARB_INLINE void
+_arb_vec_printn(arb_srcptr vec, slong len, slong ndigits, ulong flags)
+{
+    slong i;
+    for (i = 0; i < len; i++)
+    {
+        arb_printn(vec + i, ndigits, flags);
+        if (i < len - 1)
+            flint_printf(", ");
+    }
+}
+
 ARB_INLINE void
 _arb_vec_zero(arb_ptr A, slong n)
 {
@@ -876,36 +792,7 @@ _arb_vec_scalar_addmul(arb_ptr res, arb_srcptr vec,
         arb_addmul(res + i, vec + i, c, prec);
 }
 
-ARB_INLINE void
-_arb_vec_dot(arb_t res, arb_srcptr vec1, arb_srcptr vec2, slong len2, slong prec)
-{
-    slong i;
-    arb_zero(res);
-    for (i = 0; i < len2; i++)
-        arb_addmul(res, vec1 + i, vec2 + i, prec);
-}
-
-ARB_INLINE void
-_arb_vec_get_mag(mag_t bound, arb_srcptr vec, slong len)
-{
-    if (len < 1)
-    {
-        mag_zero(bound);
-    }
-    else
-    {
-        mag_t t;
-        slong i;
-        arb_get_mag(bound, vec);
-        mag_init(t);
-        for (i = 1; i < len; i++)
-        {
-            arb_get_mag(t, vec + i);
-            mag_max(bound, bound, t);
-        }
-        mag_clear(t);
-    }
-}
+void _arb_vec_get_mag(mag_t bound, arb_srcptr vec, slong len);
 
 ARB_INLINE slong
 _arb_vec_bits(arb_srcptr x, slong len)
@@ -922,23 +809,7 @@ _arb_vec_bits(arb_srcptr x, slong len)
     return b;
 }
 
-ARB_INLINE void
-_arb_vec_set_powers(arb_ptr xs, const arb_t x, slong len, slong prec)
-{
-    slong i;
-
-    for (i = 0; i < len; i++)
-    {
-        if (i == 0)
-            arb_one(xs + i);
-        else if (i == 1)
-            arb_set_round(xs + i, x, prec);
-        else if (i % 2 == 0)
-            arb_mul(xs + i, xs + i / 2, xs + i / 2, prec);
-        else
-            arb_mul(xs + i, xs + i - 1, x, prec);
-    }
-}
+void _arb_vec_set_powers(arb_ptr xs, const arb_t x, slong len, slong prec);
 
 ARB_INLINE void
 _arb_vec_add_error_arf_vec(arb_ptr res, arf_srcptr err, slong len)
@@ -1050,17 +921,18 @@ void _arb_exp_taylor_rs(mp_ptr y, mp_limb_t * error,
     mp_srcptr x, mp_size_t xn, ulong N);
 
 void arb_exp_arf_bb(arb_t z, const arf_t x, slong prec, int minus_one);
+void arb_exp_arf_rs_generic(arb_t res, const arf_t x, slong prec, int minus_one);
 
 int _arb_get_mpn_fixed_mod_log2(mp_ptr w, fmpz_t q, mp_limb_t * error,
                                                 const arf_t x, mp_size_t wn);
 
 slong _arb_exp_taylor_bound(slong mag, slong prec);
 
-void _arb_exp_sum_bs_powtab(fmpz_t T, fmpz_t Q, mp_bitcnt_t * Qexp,
-    const fmpz_t x, mp_bitcnt_t r, slong N);
+void _arb_exp_sum_bs_powtab(fmpz_t T, fmpz_t Q, flint_bitcnt_t * Qexp,
+    const fmpz_t x, flint_bitcnt_t r, slong N);
 
-void _arb_exp_sum_bs_simple(fmpz_t T, fmpz_t Q, mp_bitcnt_t * Qexp,
-    const fmpz_t x, mp_bitcnt_t r, slong N);
+void _arb_exp_sum_bs_simple(fmpz_t T, fmpz_t Q, flint_bitcnt_t * Qexp,
+    const fmpz_t x, flint_bitcnt_t r, slong N);
 
 /* sin/cos implementation */
 
@@ -1095,12 +967,22 @@ void _arb_sin_cos_taylor_rs(mp_ptr ysin, mp_ptr ycos,
 int _arb_get_mpn_fixed_mod_pi4(mp_ptr w, fmpz_t q, int * octant,
     mp_limb_t * error, const arf_t x, mp_size_t wn);
 
-ARB_INLINE mp_bitcnt_t
+void arb_sin_cos_arf_bb(arb_t zsin, arb_t zcos, const arf_t x, slong prec);
+void arb_sin_cos_arf_rs_generic(arb_t res_sin, arb_t res_cos, const arf_t x, slong prec);
+void arb_sin_cos_arf_generic(arb_t res_sin, arb_t res_cos, const arf_t x, slong prec);
+
+void _arb_sin_cos_wide(arb_t s, arb_t c, const arf_t x, const mag_t r, slong prec);
+void arb_sin_cos_wide(arb_t s, arb_t c, const arb_t x, slong prec);
+
+void _arb_sin_cos_generic(arb_t s, arb_t c, const arf_t x, const mag_t xrad, slong prec);
+void arb_sin_cos_generic(arb_t s, arb_t c, const arb_t x, slong prec);
+
+ARB_INLINE flint_bitcnt_t
 _arb_mpn_leading_zeros(mp_srcptr d, mp_size_t n)
 {
     mp_limb_t t;
     mp_size_t zero_limbs;
-    mp_bitcnt_t bits;
+    flint_bitcnt_t bits;
 
     zero_limbs = 0;
 
@@ -1121,11 +1003,11 @@ _arb_mpn_leading_zeros(mp_srcptr d, mp_size_t n)
     }
 }
 
-void _arb_atan_sum_bs_simple(fmpz_t T, fmpz_t Q, mp_bitcnt_t * Qexp,
-    const fmpz_t x, mp_bitcnt_t r, slong N);
+void _arb_atan_sum_bs_simple(fmpz_t T, fmpz_t Q, flint_bitcnt_t * Qexp,
+    const fmpz_t x, flint_bitcnt_t r, slong N);
 
-void _arb_atan_sum_bs_powtab(fmpz_t T, fmpz_t Q, mp_bitcnt_t * Qexp,
-    const fmpz_t x, mp_bitcnt_t r, slong N);
+void _arb_atan_sum_bs_powtab(fmpz_t T, fmpz_t Q, flint_bitcnt_t * Qexp,
+    const fmpz_t x, flint_bitcnt_t r, slong N);
 
 void arb_atan_arf_bb(arb_t z, const arf_t x, slong prec);
 
@@ -1161,6 +1043,14 @@ _arb_vec_estimate_allocated_bytes(slong len, slong prec)
     return size;
 }
 
+int arb_load_str(arb_t res, const char * data);
+
+char * arb_dump_str(const arb_t x);
+
+int arb_load_file(arb_t res, FILE *stream);
+
+int arb_dump_file(FILE* stream, const arb_t x);
+
 #ifdef __cplusplus
 }
 #endif
