@@ -19,6 +19,22 @@
 extern "C" {
 #endif
 
+void _arb_hypgeom_rising_coeffs_1(ulong * c, ulong k, slong l);
+void _arb_hypgeom_rising_coeffs_2(ulong * c, ulong k, slong l);
+void _arb_hypgeom_rising_coeffs_fmpz(fmpz * c, ulong k, slong l);
+
+void arb_hypgeom_rising_ui_forward(arb_t res, const arb_t x, ulong n, slong prec);
+void arb_hypgeom_rising_ui_rs(arb_t res, const arb_t x, ulong n, ulong m, slong prec);
+void arb_hypgeom_rising_ui_bs(arb_t res, const arb_t x, ulong n, slong prec);
+void arb_hypgeom_rising_ui_rec(arb_t res, const arb_t x, ulong n, slong prec);
+void arb_hypgeom_rising_ui(arb_t y, const arb_t x, ulong n, slong prec);
+void arb_hypgeom_rising(arb_t y, const arb_t x, const arb_t n, slong prec);
+
+void arb_hypgeom_rising_ui_jet_powsum(arb_ptr res, const arb_t x, ulong n, slong len, slong prec);
+void arb_hypgeom_rising_ui_jet_rs(arb_ptr res, const arb_t x, ulong n, ulong m, slong len, slong prec);
+void arb_hypgeom_rising_ui_jet_bs(arb_ptr res, const arb_t x, ulong n, slong len, slong prec);
+void arb_hypgeom_rising_ui_jet(arb_ptr res, const arb_t x, ulong n, slong len, slong prec);
+
 void arb_hypgeom_pfq(arb_t res, arb_srcptr a, slong p, arb_srcptr b, slong q,
     const arb_t z, int regularized, slong prec);
 
@@ -74,6 +90,9 @@ void arb_hypgeom_bessel_jy(arb_t res1, arb_t res2, const arb_t nu, const arb_t z
 void arb_hypgeom_bessel_i(arb_t res, const arb_t nu, const arb_t z, slong prec);
 void arb_hypgeom_bessel_k(arb_t res, const arb_t nu, const arb_t z, slong prec);
 
+void arb_hypgeom_bessel_i_scaled(arb_t res, const arb_t nu, const arb_t z, slong prec);
+void arb_hypgeom_bessel_k_scaled(arb_t res, const arb_t nu, const arb_t z, slong prec);
+
 void arb_hypgeom_airy(arb_t ai, arb_t aip, arb_t bi, arb_t bip, const arb_t z, slong prec);
 void arb_hypgeom_airy_jet(arb_ptr ai, arb_ptr bi, const arb_t z, slong len, slong prec);
 void arb_hypgeom_airy_series(arb_poly_t ai, arb_poly_t ai_prime,
@@ -81,6 +100,13 @@ void arb_hypgeom_airy_series(arb_poly_t ai, arb_poly_t ai_prime,
 void _arb_hypgeom_airy_series(arb_ptr ai, arb_ptr ai_prime,
     arb_ptr bi, arb_ptr bi_prime, arb_srcptr z, slong zlen, slong len, slong prec);
 
+void arb_hypgeom_airy_zero(arb_t ai, arb_t aip, arb_t bi, arb_t bip, const fmpz_t n, slong prec);
+
+void arb_hypgeom_coulomb(arb_t F, arb_t G, const arb_t l, const arb_t eta, const arb_t z, slong prec);
+void arb_hypgeom_coulomb_jet(arb_ptr F, arb_ptr G, const arb_t l, const arb_t eta, const arb_t z, slong len, slong prec);
+void _arb_hypgeom_coulomb_series(arb_ptr F, arb_ptr G, const arb_t l, const arb_t eta, arb_srcptr z, slong zlen, slong len, slong prec);
+void arb_hypgeom_coulomb_series(arb_poly_t F, arb_poly_t G, const arb_t l, const arb_t eta, const arb_poly_t z, slong len, slong prec);
+
 void arb_hypgeom_expint(arb_t res, const arb_t s, const arb_t z, slong prec);
 
 void arb_hypgeom_gamma_lower(arb_t res, const arb_t s, const arb_t z, int regularized, slong prec);
@@ -115,6 +141,8 @@ void arb_hypgeom_legendre_p_ui_root(arb_t res, arb_t weight, ulong n, ulong k, s
 
 void arb_hypgeom_central_bin_ui(arb_t res, ulong n, slong prec);
 
+void arb_hypgeom_dilog(arb_t res, const arb_t z, slong prec);
+
 #ifdef __cplusplus
 }
 #endif
